import click
from stairs import get_project


@click.group()
def consumer_cli():
    pass


@consumer_cli.command("consumer:standalone")
@click.argument('name')
@click.option('--noprint', '-np', is_flag=True, help="Disable print")
def init_session(name, noprint):
    project = get_project()
    project.set_verbose(not noprint)

    if project.verbose:
        print("Standalone consumer started.")

    app_name, consumer_name = name.split('.')
    user_app = project.get_app_by_name(app_name)
    user_app.components.consumers[consumer_name].run_worker()




from .app_config import app

from .flows import FlowSample
from .consumers import print_smth

from stairs import PipelineInfo, DataPoint, DataFrame


@app.pipeline()
def main(pipeline: PipelineInfo, data_frame: DataPoint) -> DataFrame:
    """
    Pipeline which apply some functions (flows) to your data.

    More info: http://stairspy.com/#pipeline

    """
    return data_frame\
        .rename(data=data_frame)\
        .subscribe_flow(FlowSample())\
        .subscribe_consumer(print_smth)






import click
from stairs import get_project
from multiprocessing import Process

@click.group()
def workers_cli():
    pass


@workers_cli.command("pipelines:run")
@click.argument('apps', nargs=-1)
@click.argument('concurrency', nargs=1, default=1)
@click.option('--noprint', '-np', is_flag=True, help="Disable print")
def run(apps, noprint, concurrency):
    """
    Run workers process
    """
    project = get_project()
    project.set_verbose(not noprint)

    if project.verbose:
        print("Pipelines started")

    processes = []
    for i in range(concurrency):
        p = Process(target=project.run_pipelines)
        p.start()
        processes.append(p)

    for p in processes:
        p.join()



from .app_config import app
from . import pipelines


@app.producer(pipelines.main)
def generate_data():
    """
    Simple producer which yields data to `main` pipeline.
    More information here: http://stairspy.com/#producer
    """
    for i in range(20):
        yield dict(data_frame=i)



import click
from stairs import get_project


@click.group()
def producer_cli():
    pass


@producer_cli.command("producer:init_session")
@click.argument('name')
@click.option('--noprint', '-np', is_flag=True, help="Disable print")
def init_session(name, noprint):
    project = get_project()
    project.set_verbose(not noprint)

    if project.verbose:
        print("Init producer session")

    if '.' in name:
        app_name, producer_name = name.split('.')
        user_app = project.get_app_by_name(app_name)
        user_app.components.producers[producer_name]()
    else:
        producer_component = None
        for app in get_project().apps:
            if name in app.components.producers:
                if producer_component is not None:
                    print("There is more then one `%s` producer found, "
                          "please specified app name: app.producer_name")
                    return
                else:
                    producer_component = app.components.producers[name]

        producer_component()


@producer_cli.command("producer:process")
@click.argument("name")
@click.option('--noprint', '-np', is_flag=True, help="Disable print")
def process(name, noprint):
    project = get_project()
    project.set_verbose(not noprint)

    if project.verbose:
        print("Producer started.")

    if '.' in name:
        app_name, producer_name = name.split('.')
        user_app = project.get_app_by_name(app_name)
        user_app.components.producers[producer_name].process()
    else:
        producer_component = None
        for app in get_project().apps:
            if name in app.components.producers:
                if producer_component is not None:
                    print("There is more then one `%s` producer found, "
                          "please specified app name: app.producer_name")
                    return
                else:
                    producer_component = app.components.producers[name]

        producer_component.process()



from stairs.core.flow import Flow, step


class FlowSample(Flow):
    """
    It's a way to run multiple steps at once.

    More info: http://stairspy.com/#flow
    """
    def __call__(self, data: dict) -> dict:
        # starts flow execution for `self.first_step`
        r = self.start_from(self.first_step, value=data)

        return dict(output=r.second_step)

    @step(None)
    def second_step(self, value2) -> dict:
        """
        Result of this step will be return to "caller".
        """
        return dict(value3=value2)

    @step(second_step)
    def first_step(self, value) -> dict:
        """
        Result of this step will be forwarded to `second_step`
        """
        return dict(value2=value)
